Package: canvasXpress
Version: 1.59.5
Title: Visualization Package for CanvasXpress in R
Description: Enables creation of visualizations using the CanvasXpress framework
    in R. CanvasXpress is a standalone JavaScript library for reproducible research
    with complete tracking of data and end-user modifications stored in a single
    PNG image that can be played back. See <https://www.canvasxpress.org> for more
    information.
Type: Package
License: GPL-3
Encoding: UTF-8
Language: en-US
Authors@R: c(
    person("Isaac", "Neuhaus", email = "imnphd@gmail.com", role = c("aut")),
    person("Connie", "Brett",  email = "connie@aggregate-genius.com", role = c("aut", "cre")))
URL: https://github.com/neuhausi/canvasXpress
BugReports: https://github.com/neuhausi/canvasXpress/issues
Depends: R (>= 3.6)
Imports: htmlwidgets (>= 1.0), htmltools, httr, jsonlite, stats
RoxygenNote: 7.3.3
Suggests: shiny (>= 1.1.0), canvasXpress.data, dplyr, DT, glue, grid,
        knitr, png, readr, rlang, rmarkdown, stringr, testthat, tibble,
        tidyr, limma, ggplot2, survminer, S7, patchwork, GGally,
        ggpubr, ggpattern
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-01-14 20:41:08 UTC; conniebrett
Author: Isaac Neuhaus [aut],
  Connie Brett [aut, cre]
Maintainer: Connie Brett <connie@aggregate-genius.com>
Repository: CRAN
Date/Publication: 2026-01-14 21:00:02 UTC
Built: R 4.6.0; ; 2026-04-26 16:56:37 UTC; unix
